import 'dart:io' as io;
import 'dart:io';
import 'dart:typed_data';
import 'package:path/path.dart' as path;

import '../utils/logger.dart';

const libext = "/lib/langauge";
const config_file_name = "/config.json";
const instruction_file_name = "/instruction.txt";
const instruction_file_path = 'src/static_files/instructions.txt';
const packages_hidden_file_path = "/.packages";
const my_package_name = "languagepro";

mixin FileSupportingMethods {
  //pending will start tommorrow
  _getPackagePath() async {
    try {
      List<FileSystemEntity> _folders;
      final myDir = await new Directory(path.current);
      File file = await File(path.current + packages_hidden_file_path);
      List<String> list_of_packages = await file.readAsLines();
      String package_string = await list_of_packages
          .firstWhere((element) => element.contains(my_package_name));
      String temppath = package_string.split("file:").last;
      String packagepath = temppath.replaceAll("///", "");

      if (packagepath.contains('lib')) {
        return Directory.current.path + "/";
      }

      return "/" + packagepath;
    } catch (e) {
      Log.print("unable to create Instructions");
      Log.print(e);
    }
  }
}

class FileOperations with FileSupportingMethods {
  Future<void> init() async {
    String languageDirectory = await _createDirectoryIfNotExist();
    String languageConfigPath =
        await _createInstructionIfNotExist(languageDirectory);
  }

  /// Create language directory if not exist
  _createDirectoryIfNotExist() async {
    final Directory langDirectory = await Directory('${path.current + libext}');
    if (await langDirectory.exists()) {
      return langDirectory.path;
    } else {
      await langDirectory.create(recursive: true).then((value) {});
    }
    return langDirectory.path;
  }

  _createConfigPathIfNotExist(String languageDirectory) async {
    File? file;
    bool file_exist = await File(languageDirectory + config_file_name).exists();
    if (file_exist == false) {
      file = await File(languageDirectory + config_file_name)
          .create(recursive: true);
    } else {
      file = await File(languageDirectory + config_file_name);
    }
    if (!file_exist) {
      file.writeAsString('''{
  "language_support": [
    {
      "english": "en",
      "hindi": "hn"
    }
  ]
}''');
    } else {
      Log.error("Config File already Exist");
    }

    return file.path;
  }

  _createInstructionIfNotExist(String languageDirectory) async {
    File sourceFile = File(await _getPackagePath() + instruction_file_path);
    File destinationfile =
        await File(languageDirectory + instruction_file_name);

    Log.print("source==> ${sourceFile.path}");
    Log.print(destinationfile.path);

    if (!(await destinationfile.exists())) {
      destinationfile.writeAsString(await sourceFile.readAsString());
    } else {
      Log.error("Config File already Exist");
    }

    return destinationfile.path;
  }
}
